"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Create = void 0;
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const setup_1 = require("@salesforce/lwc-dev-mobile-core/lib/cli/commands/force/lightning/local/setup");
const AndroidUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/AndroidUtils");
const Common_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/Common");
const CommonUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/CommonUtils");
const IOSUtils_1 = require("@salesforce/lwc-dev-mobile-core/lib/common/IOSUtils");
const util_1 = __importDefault(require("util"));
core_1.Messages.importMessagesDirectory(__dirname);
const messages = core_1.Messages.loadMessages('@salesforce/lwc-dev-mobile', 'device-create');
class Create extends setup_1.Setup {
    constructor() {
        super(...arguments);
        this.examples = [
            `sfdx force:lightning:local:device:create -p iOS -n MyNewVirtualDevice -d iPhone-8`,
            `sfdx force:lightning:local:device:create -p Android -n MyNewVirtualDevice -d pixel_xl`
        ];
        this.platform = '';
        this.deviceName = '';
        this.deviceType = '';
    }
    async run(direct = false) {
        if (direct) {
            await this.init();
        }
        this.logger.info(`Device Create command invoked for ${this.flags.platform}`);
        const extraReqs = [
            new DeviceNameAvailableRequirement(this, this.logger),
            new ValidDeviceTypeRequirement(this, this.logger)
        ];
        this.addAdditionalRequirements(extraReqs);
        return super
            .run(direct)
            .then(() => {
            this.logger.info('Setup requirements met, continuing with Device Create');
            CommonUtils_1.CommonUtils.startCliAction('Device Create', 'Creating new device');
            return this.executeDeviceCreate();
        })
            .then(() => {
            const message = util_1.default.format(messages.getMessage('success'), this.deviceName, this.deviceType);
            CommonUtils_1.CommonUtils.stopCliAction(message);
        })
            .catch((error) => {
            CommonUtils_1.CommonUtils.stopCliAction('Error encountered during device create');
            this.logger.warn(`Device Create failed for ${this.flags.platform}.`);
            return Promise.reject(error);
        });
    }
    async validateInputParameters() {
        return super.validateInputParameters().then(() => {
            const deviceName = this.flags.devicename;
            const deviceType = this.flags.devicetype;
            if (deviceName == null || deviceName.trim() === '') {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidDeviceNameFlagsDescription'), 'lwc-dev-mobile', this.examples));
            }
            if (deviceType == null || deviceType.trim() === '') {
                return Promise.reject(new core_1.SfdxError(messages.getMessage('error:invalidDeviceTypeFlagsDescription'), 'lwc-dev-mobile', this.examples));
            }
            this.platform = this.flags.platform;
            this.deviceName = deviceName;
            this.deviceType = deviceType;
            return Promise.resolve();
        });
    }
    async init() {
        if (this.logger) {
            return Promise.resolve();
        }
        return super
            .init()
            .then(() => core_1.Logger.child('force:lightning:local:device:create', {}))
            .then((logger) => {
            this.logger = logger;
            return Promise.resolve();
        });
    }
    async executeDeviceCreate() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.platform);
        return isAndroid
            ? this.executeAndroidDeviceCreate()
            : this.executeIOSDeviceCreate();
    }
    async executeAndroidDeviceCreate() {
        return AndroidUtils_1.AndroidUtils.fetchSupportedEmulatorImagePackage(this.flags.apilevel).then((preferredPack) => {
            const emuImage = preferredPack.platformEmulatorImage || 'default';
            const androidApi = preferredPack.platformAPI;
            const abi = preferredPack.abi;
            return AndroidUtils_1.AndroidUtils.createNewVirtualDevice(this.deviceName, emuImage, androidApi, this.deviceType, abi);
        });
    }
    async executeIOSDeviceCreate() {
        return IOSUtils_1.IOSUtils.getSupportedRuntimes().then((supportedRuntimes) => IOSUtils_1.IOSUtils.createNewDevice(this.deviceName, this.deviceType, supportedRuntimes[0]));
    }
}
exports.Create = Create;
Create.description = messages.getMessage('commandDescription');
Create.flagsConfig = {
    apilevel: command_1.flags.string({
        char: 'a',
        description: messages.getMessage('apiLevelFlagDescription'),
        longDescription: messages.getMessage('apiLevelFlagDescription'),
        required: false
    }),
    devicename: command_1.flags.string({
        char: 'n',
        description: messages.getMessage('deviceNameFlagDescription'),
        required: true
    }),
    devicetype: command_1.flags.string({
        char: 'd',
        description: messages.getMessage('deviceTypeFlagDescription'),
        required: true
    }),
    platform: command_1.flags.string({
        char: 'p',
        description: messages.getMessage('platformFlagDescription'),
        required: true
    })
};
class DeviceNameAvailableRequirement {
    constructor(owner, logger) {
        this.title = messages.getMessage('reqs:devicename:title');
        this.fulfilledMessage = messages.getMessage('reqs:devicename:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:devicename:unfulfilledMessage');
        this.owner = owner;
        this.logger = logger;
    }
    async checkFunction() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.owner.platform);
        const deviceName = this.owner.deviceName;
        const deviceAlreadyExists = isAndroid
            ? await AndroidUtils_1.AndroidUtils.hasEmulator(deviceName)
            : (await IOSUtils_1.IOSUtils.getSimulator(deviceName)) != null;
        return deviceAlreadyExists
            ? Promise.reject(util_1.default.format(this.unfulfilledMessage, deviceName))
            : Promise.resolve(util_1.default.format(this.fulfilledMessage, deviceName));
    }
}
class ValidDeviceTypeRequirement {
    constructor(owner, logger) {
        this.title = messages.getMessage('reqs:devicetype:title');
        this.fulfilledMessage = messages.getMessage('reqs:devicetype:fulfilledMessage');
        this.unfulfilledMessage = messages.getMessage('reqs:devicetype:unfulfilledMessage');
        this.owner = owner;
        this.logger = logger;
    }
    async checkFunction() {
        const isAndroid = Common_1.CommandLineUtils.platformFlagIsAndroid(this.owner.platform);
        const deviceType = this.owner.deviceType;
        const supportedDevices = isAndroid
            ? await AndroidUtils_1.AndroidUtils.getSupportedDevices()
            : await IOSUtils_1.IOSUtils.getSupportedDevices();
        const match = supportedDevices.find((device) => device === deviceType);
        return match !== undefined
            ? Promise.resolve(util_1.default.format(this.fulfilledMessage, deviceType))
            : Promise.reject(util_1.default.format(this.unfulfilledMessage, deviceType, supportedDevices.join(', ')));
    }
}
//# sourceMappingURL=create.js.map